@@ -21,11 +21,14 @@
 #include "agentmanager.h"
 
 #include "agentmanageradaptor.h"
+#include "agentmanagerinternaladaptor.h"
 #include "processcontrol.h"
 #include "serverinterface.h"
-#include "../../libs/xdgbasedirs_p.h"
+#include "libs/xdgbasedirs_p.h"
 #include "akdebug.h"
 #include "resource_manager.h"
+#include "preprocessor_manager.h"
+#include "libs/protocol_p.h"
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
@@ -35,6 +38,8 @@
 #include <QtDBus/QDBusError>
 #include <QtCore/QDebug>
 
+#include <boost/scoped_ptr.hpp>
+
 using Akonadi::ProcessControl;
 
 AgentManager::AgentManager( QObject *parent )
@@ -42,6 +47,7 @@ AgentManager::AgentManager( QObject *parent )
   mAgentWatcher( new QFileSystemWatcher( this ) )
 {
   new AgentManagerAdaptor( this );
+  new AgentManagerInternalAdaptor( this );
   QDBusConnection::sessionBus().registerObject( "/AgentManager", this );
 
   mTracer = new org::freedesktop::Akonadi::Tracer( "org.freedesktop.Akonadi", "/tracing", QDBusConnection::sessionBus(), this );
@@ -50,11 +56,10 @@ AgentManager::AgentManager( QObject *parent )
            this, SLOT( serviceOwnerChanged( const QString&, const QString&, const QString& ) ) );
 
   if ( QDBusConnection::sessionBus().interface()->isServiceRegistered( "org.freedesktop.Akonadi" ) )
-    qFatal( "akonadiserver already running!" );
+    akFatal() << "akonadiserver already running!";
 
   mStorageController = new Akonadi::ProcessControl;
-  connect( mStorageController, SIGNAL(unableToStart()),
-           QCoreApplication::instance(), SLOT(quit()) );
+  connect( mStorageController, SIGNAL(unableToStart()), SLOT(serverFailure()) );
   mStorageController->start( "akonadiserver", QStringList(), Akonadi::ProcessControl::RestartOnCrash );
 
   connect( mAgentWatcher, SIGNAL(fileChanged(QString)), SLOT(agentExeChanged(QString)) );
@@ -85,6 +90,13 @@ void AgentManager::continueStartup()
   load();
   foreach ( const AgentType &info, mAgents )
     ensureAutoStart( info );
+
+  // register the real service name once everything is up an running
+  if ( !QDBusConnection::sessionBus().registerService( AKONADI_DBUS_CONTROL_SERVICE ) ) {
+    // besides a race with an older Akonadi server I have no idea how we could possibly get here...
+    akFatal() << "Unable to register service as" << AKONADI_DBUS_CONTROL_SERVICE << "despite having the lock. Error was:" << QDBusConnection::sessionBus().lastError().message();
+  }
+  akDebug() << "Akonadi server is now operational.";
 }
 
 AgentManager::~AgentManager()
@@ -114,18 +126,20 @@ QStringList AgentManager::agentTypes() const
   return mAgents.keys();
 }
 
-QString AgentManager::agentName( const QString &identifier ) const
+QString AgentManager::agentName(  const QString &identifier, const QString& lang ) const
 {
   if ( !checkAgentExists( identifier ) )
     return QString();
-  return mAgents.value( identifier ).name;
+  const QString name = mAgents.value( identifier ).name.value( lang );
+  return name.isEmpty() ? mAgents.value( identifier ).name.value( "en_US" ) : name;
 }
 
-QString AgentManager::agentComment( const QString &identifier ) const
+QString AgentManager::agentComment( const QString &identifier, const QString& lang ) const
 {
   if ( !checkAgentExists( identifier ) )
     return QString();
-  return mAgents.value( identifier ).comment;
+  const QString comment = mAgents.value( identifier ).comment.value( lang );
+  return comment.isEmpty() ? mAgents.value( identifier ).comment.value( "en_US" ) : comment;
 }
 
 QString AgentManager::agentIcon( const QString &identifier ) const
@@ -172,14 +186,18 @@ QString AgentManager::createAgentInstance( const QString &identifier )
     return QString();
   }
 
+  // Return from this dbus call before we do the next. Otherwise dbus brakes for
+  // this process.
+  if ( calledFromDBus() )
+    connection().send( message().createReply( instance->identifier() ) );
+
   if ( !instance->start( agentInfo ) )
     return QString();
+
   mAgentInstances.insert( instance->identifier(), instance );
-
-  org::freedesktop::Akonadi::ResourceManager * resmanager = new org::freedesktop::Akonadi::ResourceManager( QLatin1String("org.freedesktop.Akonadi"), QLatin1String("/ResourceManager"), QDBusConnection::sessionBus(), this );
-  resmanager->addResourceInstance(instance->identifier());
-
+  registerAgentAtServer( instance, agentInfo );
   save();
+
   return instance->identifier();
 }
 
@@ -206,6 +224,16 @@ void AgentManager::removeAgentInstance( const QString &identifier )
   org::freedesktop::Akonadi::ResourceManager * resmanager = new org::freedesktop::Akonadi::ResourceManager( QLatin1String("org.freedesktop.Akonadi"), QLatin1String("/ResourceManager"), QDBusConnection::sessionBus(), this );
   resmanager->removeResourceInstance(instance->identifier());
 
+  // Kill the preprocessor instance, if any.
+  org::freedesktop::Akonadi::PreprocessorManager preProcessorManager(
+      QLatin1String( "org.freedesktop.Akonadi" ),
+      QLatin1String( "/PreprocessorManager" ),
+      QDBusConnection::sessionBus(),
+      this
+    );
+
+  preProcessorManager.unregisterInstance( instance->identifier() );
+
   emit agentInstanceRemoved( identifier );
 }
 
@@ -282,7 +310,7 @@ void AgentManager::setAgentInstanceName( const QString &identifier, const QStrin
   mAgentInstances.value( identifier )->resourceInterface()->setName( name );
 }
 
-QString AgentManager::agentInstanceName( const QString &identifier ) const
+QString AgentManager::agentInstanceName( const QString &identifier, const QString &lang ) const
 {
   if ( !checkInstance( identifier ) )
     return QString();
@@ -291,7 +319,8 @@ QString AgentManager::agentInstanceName( const QString &identifier ) const
     return inst->resourceName();
   if ( !checkAgentExists( inst->agentType() ) )
     return QString();
-  return mAgents.value( inst->agentType() ).name;
+  const QString name = mAgents.value( inst->agentType() ).name.value( lang );
+  return name.isEmpty() ? mAgents.value( inst->agentType() ).name.value("en_US") : name;
 }
 
 void AgentManager::agentInstanceSynchronize( const QString &identifier )
@@ -367,12 +396,26 @@ void AgentManager::readPluginInfos( const QDir& directory )
       if ( mAgents.contains( agentInfo.identifier ) ) {
         mTracer->error( QLatin1String( "AgentManager::updatePluginInfos" ),
                         QString( "Duplicated agent identifier '%1' from file '%2'" )
-                            .arg( fileName, agentInfo.identifier ) );
+                            .arg( agentInfo.identifier, fileName ) );
         continue;
       }
+
+      const QString disableAutostart = getEnv( "AKONADI_DISABLE_AGENT_AUTOSTART" );
+      if( !disableAutostart.isEmpty() ) {
+        qDebug() << "Autostarting of agents is disabled.";
+        agentInfo.capabilities.removeOne( AgentType::CapabilityAutostart );
+      }
+
+      const QString executable = Akonadi::XdgBaseDirs::findExecutableFile( agentInfo.exec );
+      if ( executable.isEmpty() ) {
+        akError() << "Executable" << agentInfo.exec << "for agent" << agentInfo.identifier << "could not be found!";
+        continue;
+      }
+
       qDebug() << "PLUGINS inserting: " << agentInfo.identifier << agentInfo.instanceCounter << agentInfo.capabilities;
       mAgents.insert( agentInfo.identifier, agentInfo );
-      mAgentWatcher->addPath( Akonadi::XdgBaseDirs::findExecutableFile( agentInfo.exec ) );
+      if ( !mAgentWatcher->files().contains( executable) )
+	mAgentWatcher->addPath( executable );
     }
   }
 }
@@ -450,16 +493,24 @@ void AgentManager::save()
   file.sync();
 }
 
-void AgentManager::serviceOwnerChanged( const QString &name, const QString&, const QString &newOwner )
+void AgentManager::serviceOwnerChanged( const QString &name, const QString &, const QString &newOwner )
 {
+  // This is called by the D-Bus server when a service comes up, goes down or changes ownership for some reason
+  // and this is where we "hook up" our different Agent interfaces.
+
+  //qDebug() << "Service " << name << " owner changed from " << oldOwner << " to " << newOwner;
+
   if ( name == "org.freedesktop.Akonadi" && !newOwner.isEmpty() ) {
     // server is operational, start agents
     continueStartup();
   }
 
-  if ( name.startsWith( "org.freedesktop.Akonadi.Agent." ) ) {
+  if ( name.startsWith( QLatin1String("org.freedesktop.Akonadi.Agent.") ) ) {
+
+    // An agent service went up or down
+
     if ( newOwner.isEmpty() )
-      return;
+      return; // It went down: we don't care here.
 
     const QString identifier = name.mid( 30 );
     if ( !mAgentInstances.contains( identifier ) )
@@ -474,9 +525,12 @@ void AgentManager::serviceOwnerChanged( const QString &name, const QString&, con
       emit agentInstanceAdded( identifier );
   }
 
-  else if ( name.startsWith( "org.freedesktop.Akonadi.Resource." ) ) {
+  else if ( name.startsWith( QLatin1String("org.freedesktop.Akonadi.Resource.") ) ) {
+
+    // A resource service went up or down
+
     if ( newOwner.isEmpty() )
-      return;
+      return; // It went down: we don't care here.
 
     const QString identifier = name.mid( 33 );
     if ( !mAgentInstances.contains( identifier ) )
@@ -484,6 +538,75 @@ void AgentManager::serviceOwnerChanged( const QString &name, const QString&, con
 
     mAgentInstances.value( identifier )->obtainResourceInterface();
   }
+
+  else if ( name.startsWith( QLatin1String("org.freedesktop.Akonadi.Preprocessor.") ) ) {
+
+    // A preprocessor service went up or down
+
+    // If the preprocessor is going up then the org.freedesktop.Akonadi.Agent.* interface
+    // should be already up (as it's registered before the preprocessor one).
+    // So if we don't know about the preprocessor as agent instance
+    // then it's not our preprocessor.
+
+    // If the preprocessor is going down then either the agent interface already
+    // went down (and it has been already unregistered on the manager side)
+    // or it's still registered as agent and WE have to unregister it.
+    // The order of interface deletions depends on Qt but we handle both cases.
+
+    // Check if we "know" about it.
+
+    const QString identifier = name.mid( 37 );
+
+    qDebug() << "Preprocessor " << identifier << " is going up or down...";
+
+    if ( !mAgentInstances.contains( identifier ) )
+    {
+      qDebug() << "But it isn't registered as agent... not mine (anymore?)";
+      return; // not our agent (?)
+    }
+
+    org::freedesktop::Akonadi::PreprocessorManager preProcessorManager(
+        QLatin1String( "org.freedesktop.Akonadi" ),
+        QLatin1String( "/PreprocessorManager" ),
+        QDBusConnection::sessionBus(),
+        this
+      );
+
+    if( !preProcessorManager.isValid() )
+    {
+      mTracer->warning(
+          QLatin1String( "AgentManager::serviceOwnerChanged" ),
+          QString( "Could not connect to PreprocessorManager via D-Bus: %1" )
+            .arg( preProcessorManager.lastError().message() )
+        );
+    } else {
+      if ( newOwner.isEmpty() )
+      {
+        // The preprocessor went down. Unregister it on server side.
+
+        preProcessorManager.unregisterInstance( identifier );
+
+      } else {
+
+        // The preprocessor went up. Register it on server side.
+
+        if ( !mAgentInstances.value( identifier )->obtainPreprocessorInterface() )
+        {
+          // Hm.. couldn't hook up its preprocessor interface..
+          // Make sure we don't have it in the preprocessor chain
+          qWarning() << "Couldn't obtain preprocessor interface for instance " << identifier;
+ 
+          preProcessorManager.unregisterInstance( identifier );
+          return;
+        }
+
+        qDebug() << "Registering preprocessor instance " << identifier;
+
+        // Add to the preprocessor chain
+        preProcessorManager.registerInstance( identifier );
+      }
+    }
+  }
 }
 
 bool AgentManager::checkInstance(const QString & identifier) const
@@ -541,6 +664,7 @@ void AgentManager::ensureAutoStart(const AgentType & info)
   instance->setIdentifier( info.identifier );
   if ( instance->start( info ) ) {
     mAgentInstances.insert( instance->identifier(), instance );
+    registerAgentAtServer( instance, info );
     save();
   }
 }
@@ -559,5 +683,41 @@ void AgentManager::agentExeChanged(const QString & fileName)
   }
 }
 
+void AgentManager::registerAgentAtServer( const AgentInstance::Ptr &instance, const AgentType &type )
+{
+  if ( type.capabilities.contains( AgentType::CapabilityResource ) ) {
+    boost::scoped_ptr<org::freedesktop::Akonadi::ResourceManager> resmanager(
+      new org::freedesktop::Akonadi::ResourceManager( QLatin1String("org.freedesktop.Akonadi"),
+                                                      QLatin1String("/ResourceManager"),
+                                                      QDBusConnection::sessionBus(), this ) );
+    resmanager->addResourceInstance( instance->identifier(), type.capabilities );
+  }
+}
+
+
+void AgentManager::addSearch(const QString& query, const QString& queryLanguage, qint64 resultCollectionId)
+{
+  qDebug() << "AgentManager::addSearch" << query << queryLanguage << resultCollectionId;
+  foreach ( const AgentInstance::Ptr &instance, mAgentInstances ) {
+    const AgentType type = mAgents.value( instance->agentType() );
+    if ( type.capabilities.contains( AgentType::CapabilitySearch ) && instance->searchInterface() )
+      instance->searchInterface()->addSearch( query, queryLanguage, resultCollectionId );
+  }
+}
+
+void AgentManager::removeSearch(quint64 resultCollectionId)
+{
+  qDebug() << "AgentManager::removeSearch" << resultCollectionId;
+  foreach ( const AgentInstance::Ptr &instance, mAgentInstances ) {
+    const AgentType type = mAgents.value( instance->agentType() );
+    if ( type.capabilities.contains( AgentType::CapabilitySearch ) && instance->searchInterface() )
+      instance->searchInterface()->removeSearch( resultCollectionId );
+  }
+}
+
+void AgentManager::serverFailure()
+{
+  QCoreApplication::instance()->exit( 255 );
+}
 
 #include "agentmanager.moc"
