@@ -61,7 +61,7 @@ bool Append::commit()
     if ( m_streamParser->hasLiteral() ) {
       dataSize = m_streamParser->remainingLiteralSize();
       m_size = qMax( m_size, dataSize );
-      storeInFile = DbConfig::useExternalPayloadFile() && dataSize > DbConfig::sizeThreshold();
+      storeInFile = DbConfig::configuredDatabase()->useExternalPayloadFile() && dataSize > DbConfig::configuredDatabase()->sizeThreshold();
       if ( storeInFile ) {
         if ( !tmpFile.open() ) {
           storeInFile =  false;
@@ -95,6 +95,7 @@ bool Append::commit()
 
     QByteArray mt;
     QString remote_id;
+    QString remote_revision;
     QList<QByteArray> flags;
     foreach( const QByteArray &flag, m_flags ) {
       if ( flag.startsWith( "\\MimeType" ) ) {
@@ -105,6 +106,10 @@ bool Append::commit()
         int pos1 = flag.indexOf( '[' );
         int pos2 = flag.lastIndexOf( ']' );
         remote_id = QString::fromUtf8( flag.mid( pos1 + 1, pos2 - pos1 - 1 ) );
+      } else if ( flag.startsWith( "\\RemoteRevision" ) ) {
+        int pos1 = flag.indexOf( '[' );
+        int pos2 = flag.lastIndexOf( ']' );
+        remote_revision = QString::fromUtf8( flag.mid( pos1 + 1, pos2 - pos1 - 1 ) );
       } else
         flags << flag;
     }
@@ -139,7 +144,7 @@ bool Append::commit()
     // If we have active preprocessors then we also set the hidden attribute
     // for the UI and we enqueue the item for preprocessing.
     bool doPreprocessing = PreprocessorManager::instance()->isActive();
-    qDebug() << "Append handler: doPreprocessing is" << doPreprocessing;
+    //qDebug() << "Append handler: doPreprocessing is" << doPreprocessing;
     if ( doPreprocessing )
     {
       Part hiddenAttribute;
@@ -154,7 +159,7 @@ bool Append::commit()
       parts.append( hiddenAttribute );
     }
 
-    bool ok = db->appendPimItem( parts, mimeType, col, m_dateTime, remote_id, item );
+    bool ok = db->appendPimItem( parts, mimeType, col, m_dateTime, remote_id, remote_revision, item );
     response.setTag( tag() );
     if ( !ok ) {
         return failureResponse( "Append failed" );
@@ -195,7 +200,7 @@ bool Append::commit()
     // Date time is always stored in UTC time zone by the server.
     QString datetime = QLocale::c().toString( item.datetime(), QLatin1String( "dd-MMM-yyyy hh:mm:ss +0000" ) );
 
-    QByteArray res( "[UIDNEXT " + QByteArray::number( item.id() ) + " " );
+    QByteArray res( "[UIDNEXT " + QByteArray::number( item.id() ) + ' ' );
     res.append( "DATETIME " + ImapParser::quote( datetime.toUtf8() ) );
     res.append( ']' );
 
