@@ -23,18 +23,28 @@
 #include <handler.h>
 
 #include "scope.h"
-#include "libs/imapset_p.h"
-#include "storage/datastore.h"
 
 namespace Akonadi {
 
-class ImapSet;
-class QueryBuilder;
-
 /**
   @ingroup akonadi_server_handler
 
   Handler for the fetch command.
+
+  Request syntax:
+  @verbatim
+  fetch-request = tag " " [scope-selector " "] "FETCH " scope " " fetch-parameters " " part-list
+  scope-selector = [ "UID" / "RID" ]
+  fetch-parameters = [ "FULLPAYLOAD" / "CACHEONLY" / "CACHEONLY" / "EXTERNALPAYLOAD" / "ANCESTORS " depth ]
+  part-list = "(" *(part-id) ")"
+  depth = "0" / "1" / "INF"
+  @endverbatim
+
+  Semantics:
+  - @c FULLPAYLOAD: Retrieve the full payload
+  - @c CACHEONLY: Restrict retrieval to parts already in the cache, even if more parts have been requested.
+  - @c EXTERNALPAYLOAD: Indicate the capability to retrieve parts via the filesystem instead over the socket
+  - @c ANCESTORS: Indicate the desired ancestor collection depth (0 is the default) 
  */
 class Fetch : public Handler
 {
@@ -44,25 +54,11 @@ class Fetch : public Handler
 
     bool parseStream();
 
-  private:
-    void parseCommand( const QByteArray &line );
-    void updateItemAccessTime();
-    void triggerOnDemandFetch();
-    void buildItemQuery();
-    QueryBuilder buildPartQuery( const QStringList &partList, bool allPayload, bool allAttrs );
-    void retrieveMissingPayloads( const QStringList &payloadList );
-    void parseCommandStream();
+  private Q_SLOTS:
+    void slotFailureResponse( const QString& );
 
   private:
-    QueryBuilder mItemQuery;
-    QList<QByteArray> mRequestedParts;
     Scope mScope;
-    bool mCacheOnly;
-    bool mFullPayload;
-    bool mAllAttrs;
-    bool mSizeRequested;
-    bool mMTimeRequested;
-    bool mExternalPayloadSupported;
 };
 
 }
