@@ -129,6 +129,7 @@ bool Store::parseStream()
 
   QSet<QByteArray> changes;
   qint64 partSizes = 0;
+  bool invalidateCache = false;
 
   // apply modifications
   m_streamParser->beginList();
@@ -197,11 +198,25 @@ bool Store::parseStream()
       }
     }
 
+    else if ( command == AKONADI_PARAM_REMOTEREVISION ) {
+      const QString remoteRevision = m_streamParser->readUtf8String();
+      if ( item.remoteRevision() != remoteRevision ) {
+        if ( !connection()->isOwnerResource( item ) )
+          throw HandlerException( "Only resources can modify remote revisions" );
+        item.setRemoteRevision( remoteRevision );
+        changes << AKONADI_PARAM_REMOTEREVISION;
+      }
+    }
+
     else if ( command == AKONADI_PARAM_UNDIRTY ) {
-      m_streamParser->readString(); // ### ???
+      m_streamParser->readString(); // read the 'false' string
       item.setDirty( false );
     }
 
+    else if ( command == AKONADI_PARAM_INVALIDATECACHE ) {
+      invalidateCache = true;
+    }
+
     else if ( command == AKONADI_PARAM_SIZE ) {
       mSize = m_streamParser->readNumber();
       changes << AKONADI_PARAM_SIZE;
@@ -244,11 +259,14 @@ bool Store::parseStream()
       if ( m_streamParser->hasLiteral() ) {
         const qint64 dataSize = m_streamParser->remainingLiteralSize();
         partSizes += dataSize;
-        const bool storeInFile = ( DbConfig::useExternalPayloadFile() && dataSize > DbConfig::sizeThreshold() );
+        const bool storeInFile = ( DbConfig::configuredDatabase()->useExternalPayloadFile() && dataSize > DbConfig::configuredDatabase()->sizeThreshold() );
         //actual case when streaming storage is used: external payload is enabled, data is big enough in a literal
         if ( storeInFile ) {
           part.setExternal( true ); //the part WILL be external
-          value = m_streamParser->readLiteralPart(); // ### why?
+          // use first part as value for the initial insert into / update to the database.
+          // this will give us a proper filename to stream the rest of the parts contents into
+          // NOTE: we have to set the correct size (== dataSize) directly
+          value = m_streamParser->readLiteralPart();
 
           if ( part.isValid() ) {
             if ( !PartHelper::update( &part, value, dataSize ) )
@@ -256,12 +274,13 @@ bool Store::parseStream()
           } else {
 //             qDebug() << "insert from Store::handleLine";
             part.setData( value );
-            part.setDatasize( value.size() ); // ### why not datasize?
+            part.setDatasize( dataSize );
             if ( !PartHelper::insert( &part ) )
               return failureResponse( "Unable to add item part" );
           }
 
-          //the actual streaming code, reads from the parser, writes immediately to the file
+          //the actual streaming code for the remaining parts:
+          // reads from the parser, writes immediately to the file
           // ### move this entire block to part helper? should be useful for append as well
           const QString fileName = QString::fromUtf8( part.data() );
           QFile file( fileName );
@@ -288,6 +307,7 @@ bool Store::parseStream()
         partSizes += value.size();
       }
 
+      // only relevant for non-literals or non-external literals
       const QByteArray origData = PartHelper::translateData( part );
       if ( origData != value ) {
         if ( part.isValid() ) {
@@ -321,6 +341,13 @@ bool Store::parseStream()
         item.setDirty( true );
       if ( !item.update() )
         throw HandlerException( "Unable to write item changes into the database" );
+
+      if ( invalidateCache ) {
+        if ( !store->invalidateItemCache( item ) ) {
+          throw HandlerException( "Unable to invalidate item cache in the database" );
+        }
+      }
+
       store->notificationCollector()->itemChanged( item, changes );
     }
 
@@ -332,6 +359,8 @@ bool Store::parseStream()
     datetime = QLocale::c().toString( pimItems.first().datetime(), QLatin1String( "dd-MMM-yyyy hh:mm:ss +0000" ) );
   }
 
+  // TODO: When implementing support for modifying multiple items at once, the revisions of the items should be in the responses.
+  // or only modified items should appear in the repsponse.
   Response response;
   response.setTag( tag() );
   response.setSuccess();
