@@ -25,6 +25,7 @@
 #include "dbupdater.h"
 #include "notificationmanager.h"
 #include "tracer.h"
+#include "transaction.h"
 #include "selectquerybuilder.h"
 #include "handlerhelper.h"
 #include "countquerybuilder.h"
@@ -52,6 +53,10 @@
 
 using namespace Akonadi;
 
+static QMutex sTransactionMutex;
+#define TRANSACTION_MUTEX_LOCK if ( m_database.driverName().startsWith( QLatin1String( "QSQLITE" ) ) ) sTransactionMutex.lock()
+#define TRANSACTION_MUTEX_UNLOCK if ( m_database.driverName().startsWith( QLatin1String( "QSQLITE" ) ) ) sTransactionMutex.unlock()
+
 /***************************************************************************
  *   DataStore                                                           *
  ***************************************************************************/
@@ -77,8 +82,8 @@ void DataStore::open()
   m_connectionName = QUuid::createUuid().toString() + QString::number( reinterpret_cast<qulonglong>( QThread::currentThread() ) );
   Q_ASSERT( !QSqlDatabase::contains( m_connectionName ) );
 
-  m_database = QSqlDatabase::addDatabase( DbConfig::driverName(), m_connectionName );
-  DbConfig::configure( m_database );
+  m_database = QSqlDatabase::addDatabase( DbConfig::configuredDatabase()->driverName(), m_connectionName );
+  DbConfig::configuredDatabase()->apply( m_database );
 
   if ( !m_database.isValid() ) {
     m_dbOpened = false;
@@ -206,6 +211,34 @@ bool DataStore::removeItemParts( const PimItem &item, const QList<QByteArray> &p
   return true;
 }
 
+bool DataStore::invalidateItemCache( const PimItem &item )
+{
+  // find all expired item parts
+  SelectQueryBuilder<Part> qb;
+  qb.addJoin( QueryBuilder::InnerJoin, PimItem::tableName(), PimItem::idFullColumnName(), Part::pimItemIdFullColumnName() );
+  qb.addValueCondition( Part::pimItemIdFullColumnName(), Query::Equals, item.id() );
+  qb.addValueCondition( Part::dataFullColumnName(), Query::IsNot, QVariant() );
+  qb.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( "PLD:" ) );
+  qb.addValueCondition( PimItem::dirtyFullColumnName(), Query::Equals, false );
+
+  if ( !qb.exec() )
+    return false;
+
+  Part::List parts = qb.result();
+  PartHelper::loadData(parts); //FIXME: not needed anymore to read back the data itself?
+
+  if ( parts.isEmpty() )
+    return true;
+
+  // clear data field
+  for ( int i = 0; i < parts.count(); ++i) {
+    if ( !PartHelper::update( &(parts[ i ]), QByteArray(), 0) )
+      return false;
+  }
+
+  return true;
+}
+
 /* --- Collection ------------------------------------------------------ */
 bool DataStore::appendCollection( Collection &collection )
 {
@@ -241,12 +274,16 @@ bool Akonadi::DataStore::cleanupCollection(Collection &collection)
 
 static bool recursiveSetResourceId( const Collection & collection, qint64 resourceId )
 {
-  QueryBuilder qb( QueryBuilder::Update );
-  qb.addTable( Collection::tableName() );
+  Transaction transaction( DataStore::self() );
+
+  QueryBuilder qb( Collection::tableName(), QueryBuilder::Update );
   qb.addValueCondition( Collection::parentIdColumn(), Query::Equals, collection.id() );
   qb.setColumnValue( Collection::resourceIdColumn(), resourceId );
   if ( !qb.exec() )
     return false;
+
+  transaction.commit();
+
   foreach ( const Collection &col, collection.children() ) {
     if ( !recursiveSetResourceId( col, resourceId ) )
       return false;
@@ -360,6 +397,7 @@ bool DataStore::appendPimItem( QList<Part> & parts,
                                const Collection & collection,
                                const QDateTime & dateTime,
                                const QString & remote_id,
+                               const QString & remoteRevision,
                                PimItem &pimItem )
 {
   pimItem.setMimeTypeId( mimetype.id() );
@@ -374,6 +412,7 @@ bool DataStore::appendPimItem( QList<Part> & parts,
     pimItem.setRemoteId( remote_id );
     pimItem.setDirty( false );
   }
+  pimItem.setRemoteRevision( remoteRevision );
   pimItem.setAtime( QDateTime::currentDateTime() );
 
   if ( !pimItem.insert() )
@@ -449,14 +488,16 @@ bool DataStore::cleanupPimItems( const Collection &collection )
   if ( !m_dbOpened || !collection.isValid() )
     return false;
 
-  QueryBuilder qb( QueryBuilder::Select );
-  qb.addTable( Flag::tableName() );
-  qb.addTable( PimItemFlagRelation::tableName() );
-  qb.addTable( PimItem::tableName() );
+  QueryBuilder qb( Flag::tableName(), QueryBuilder::Select );
+  qb.addJoin( QueryBuilder::InnerJoin, PimItemFlagRelation::tableName(),
+              PimItemFlagRelation::rightFullColumnName(), Flag::idFullColumnName() );
+  qb.addJoin( QueryBuilder::InnerJoin, PimItem::tableName(),
+              PimItem::idFullColumnName(), PimItemFlagRelation::leftFullColumnName() );
+
   qb.addColumn( PimItemFlagRelation::leftFullColumnName() );
+
   qb.addValueCondition( Flag::nameFullColumnName(), Query::Equals, QLatin1String("\\Deleted") );
   qb.addValueCondition( PimItem::collectionIdFullColumnName(), Query::Equals, collection.id() );
-  qb.addColumnCondition( PimItemFlagRelation::rightFullColumnName(), Query::Equals, Flag::idFullColumnName() );
 
   if ( !qb.exec() )
     return false;
@@ -600,9 +641,11 @@ bool Akonadi::DataStore::beginTransaction()
     return false;
 
   if ( m_transactionLevel == 0 ) {
+    TRANSACTION_MUTEX_LOCK; 
     QSqlDriver *driver = m_database.driver();
     if ( !driver->beginTransaction() ) {
       debugLastDbError( "DataStore::beginTransaction" );
+      TRANSACTION_MUTEX_UNLOCK;
       return false;
     }
   }
@@ -628,9 +671,12 @@ bool Akonadi::DataStore::rollbackTransaction()
     QSqlDriver *driver = m_database.driver();
     emit transactionRolledBack();
     if ( !driver->rollbackTransaction() ) {
+      TRANSACTION_MUTEX_UNLOCK;
       debugLastDbError( "DataStore::rollbackTransaction" );
       return false;
     }
+
+    TRANSACTION_MUTEX_UNLOCK;
   }
 
   return true;
@@ -653,6 +699,7 @@ bool Akonadi::DataStore::commitTransaction()
       rollbackTransaction();
       return false;
     } else {
+      TRANSACTION_MUTEX_UNLOCK;
       emit transactionCommitted();
     }
   }
