@@ -124,6 +124,9 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
     /* --- ItemParts ----------------------------------------------------- */
     bool removeItemParts( const PimItem &item, const QList<QByteArray> &parts );
 
+    // removes all payload parts for this item.
+    bool invalidateItemCache( const PimItem &item );
+
     /* --- Collection ------------------------------------------------------ */
     bool appendCollection( Collection &collection );
     /// removes the given collection and all its content
@@ -150,6 +153,7 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
                         const Collection & collection,
                         const QDateTime & dateTime,
                         const QString & remote_id,
+                        const QString & remoteRevision,
                         PimItem &pimItem );
 
     /**
@@ -170,7 +174,7 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
      * is actually a new item arrival).
      *
      * This function does NOT verify if the item was *really* hidden: this is
-     * responsability of the caller.
+     * responsibility of the caller.
      */
     bool unhidePimItem( PimItem &pimItem );
 
