@@ -17,8 +17,8 @@
     02110-1301, USA.
 */
 
-#ifndef AKONADI_ITEMRETRIEVERMANAGER_H
-#define AKONADI_ITEMRETRIEVERMANAGER_H
+#ifndef AKONADI_ITEMRETRIEVALMANAGER_H
+#define AKONADI_ITEMRETRIEVALMANAGER_H
 
 #include "itemretriever.h"
 
@@ -46,6 +46,13 @@ class ItemRetrievalManager : public QObject
 
     void requestItemDelivery( qint64 uid, const QByteArray& remoteId, const QByteArray& mimeType,
                               const QString &resource, const QStringList &parts );
+
+    /**
+     * Added for convenience. ItemRetrievalManager takes ownership over the
+     * pointer and deletes it when the request is processed.
+     */
+    void requestItemDelivery( ItemRetrievalRequest *request );
+
     void requestCollectionSync( const Collection &collection );
 
     static ItemRetrievalManager* instance();
@@ -53,6 +60,7 @@ class ItemRetrievalManager : public QObject
   signals:
     void requestAdded();
     void syncCollection( const QString &resource, qint64 colId );
+    void syncResource( const QString &resource );
 
   private:
     OrgFreedesktopAkonadiResourceInterface* resourceInterface( const QString &id );
@@ -61,6 +69,7 @@ class ItemRetrievalManager : public QObject
     void serviceOwnerChanged( const QString &serviceName, const QString &oldOwner, const QString &newOwner );
     void processRequest();
     void triggerCollectionSync( const QString &resource, qint64 colId );
+    void triggerResourceSync( const QString &resource );
     void retrievalJobFinished( ItemRetrievalRequest* request, const QString &errorMsg );
 
   private:
