@@ -43,7 +43,7 @@ bool PartHelper::update( Part *part, const QByteArray &data, qint64 dataSize )
   if (!part)
     return false;
 
-  if (DbConfig::useExternalPayloadFile() && part->external())
+  if (DbConfig::configuredDatabase()->useExternalPayloadFile() && part->external())
   {
     QString origFileName = QString::fromUtf8( part->data() );
     QString fileName = origFileName;
@@ -64,7 +64,7 @@ bool PartHelper::update( Part *part, const QByteArray &data, qint64 dataSize )
 
     if (file.open( QIODevice::WriteOnly | QIODevice::Truncate ))
     {
-      qDebug() << "Update part file " << fileName <<" with " << QString::fromUtf8(data).left(50);
+//      qDebug() << "Update part file " << fileName <<" with " << QString::fromUtf8(data).left(50);
 
       file.write( data );
       QByteArray fileNameData = fileName.toLocal8Bit();
@@ -72,12 +72,12 @@ bool PartHelper::update( Part *part, const QByteArray &data, qint64 dataSize )
       part->setDatasize( fileNameData.size() );
       part->setExternal( true );
       file.close();
-      qDebug() << "Removing part file " << origFileName;
+//      qDebug() << "Removing part file " << origFileName;
       QFile::remove(origFileName);
     } else
     {
-      qDebug() << "Update: payload file " << fileName << " could not be open for writing!";
-      qDebug() << "Error: " << file.errorString();
+//      qDebug() << "Update: payload file " << fileName << " could not be open for writing!";
+//      qDebug() << "Error: " << file.errorString();
       return false;
     }
   } else
@@ -97,7 +97,7 @@ bool PartHelper::insert( Part *part, qint64* insertId )
 //   qDebug() << "Insert original data " << part->data();
   QByteArray fileNameData("");
   QByteArray data;
-  bool storeInFile = DbConfig::useExternalPayloadFile()  && ( part->datasize() > DbConfig::sizeThreshold() || part->external() );
+  bool storeInFile = DbConfig::configuredDatabase()->useExternalPayloadFile()  && ( part->datasize() > DbConfig::configuredDatabase()->sizeThreshold() || part->external() );
 
   //it is needed to insert first the metadata so a new id is generated for the part,
   //and we need this id for the payload file name
@@ -123,7 +123,7 @@ bool PartHelper::insert( Part *part, qint64* insertId )
 
     if (file.open( QIODevice::WriteOnly | QIODevice::Truncate ))
     {
-      qDebug() << "Insert: create part file " << fileName << "with " << QString::fromUtf8(data).left(50);
+//      qDebug() << "Insert: create part file " << fileName << "with " << QString::fromUtf8(data).left(50);
 
       file.write(data);
       fileNameData = fileName.toLocal8Bit();
@@ -133,8 +133,8 @@ bool PartHelper::insert( Part *part, qint64* insertId )
       file.close();
     } else
     {
-      qDebug() << "Insert: payload file " << fileName << " could not be open for writing!";
-      qDebug() << "Error: " << file.errorString();
+//      qDebug() << "Insert: payload file " << fileName << " could not be open for writing!";
+//      qDebug() << "Error: " << file.errorString();
       return false;
     }
   }
@@ -146,9 +146,9 @@ bool PartHelper::remove( Akonadi::Part *part )
   if (!part)
     return false;
 
-  if (DbConfig::useExternalPayloadFile()  && part->external())
+  if (DbConfig::configuredDatabase()->useExternalPayloadFile()  && part->external())
   {
-    qDebug() << "remove part file " << part->data();
+    // qDebug() << "remove part file " << part->data();
     QString fileName = QString::fromUtf8( part->data() );
     QFile::remove( fileName );
   }
@@ -157,13 +157,13 @@ bool PartHelper::remove( Akonadi::Part *part )
 
 bool PartHelper::remove( const QString &column, const QVariant &value )
 {
-  if ( DbConfig::useExternalPayloadFile() )
+  if ( DbConfig::configuredDatabase()->useExternalPayloadFile() )
   {
     SelectQueryBuilder<Part> builder;
     builder.addValueCondition( column, Query::Equals, value );
     if ( !builder.exec() ) {
-      qDebug() << "Error selecting records to be deleted from table"
-          << Part::tableName() << builder.query().lastError().text();
+//      qDebug() << "Error selecting records to be deleted from table"
+//          << Part::tableName() << builder.query().lastError().text();
       return false;
     }
     Part::List parts = builder.result();
@@ -174,7 +174,7 @@ bool PartHelper::remove( const QString &column, const QVariant &value )
       if ((*it).external())
       {
         QString fileName = QString::fromUtf8( (*it).data() );
-        qDebug() << "remove part file " << fileName;
+        // qDebug() << "remove part file " << fileName;
         QFile::remove( fileName );
       }
     }
@@ -199,7 +199,7 @@ bool PartHelper::loadData( Part::List &parts )
 
 bool PartHelper::loadData( Part &part )
 {
-  if ( DbConfig::useExternalPayloadFile() && part.external() )
+  if ( DbConfig::configuredDatabase()->useExternalPayloadFile() && part.external() )
   {
     QString fileName = QString::fromUtf8( part.data() );
     QFile file( fileName );
@@ -208,12 +208,12 @@ bool PartHelper::loadData( Part &part )
       QByteArray data = file.readAll();
       part.setData( data );
       part.setDatasize( data.size() );
-      qDebug() << "load part file " << fileName << QString::fromUtf8(data).left(50);
+//      qDebug() << "load part file " << fileName << QString::fromUtf8(data).left(50);
       file.close();
     } else
     {
-      qDebug() << "Payload file " << fileName << " could not be open for reading!";
-      qDebug() << "Error: " << file.errorString();
+//      qDebug() << "Payload file " << fileName << " could not be open for reading!";
+//      qDebug() << "Error: " << file.errorString();
       return false;
     }
   } else
@@ -225,13 +225,10 @@ bool PartHelper::loadData( Part &part )
   return true;
 }
 
-QByteArray PartHelper::translateData( qint64 id, const QByteArray &data, bool isExternal )
+QByteArray PartHelper::translateData( const QByteArray &data, bool isExternal )
 {
-  Q_UNUSED(id);
-
-  if ( DbConfig::useExternalPayloadFile() && isExternal )
+  if ( DbConfig::configuredDatabase()->useExternalPayloadFile() && isExternal )
   {
-    //qDebug() << "translateData " << id;
     QString fileName = QString::fromUtf8( data );
     QFile file( fileName );
     if (file.open( QIODevice::ReadOnly ))
@@ -241,8 +238,8 @@ QByteArray PartHelper::translateData( qint64 id, const QByteArray &data, bool is
       return payload;
     } else
     {
-      qDebug() << "Payload file " << fileName << " could not be open for reading!";
-      qDebug() << "Error: " << file.errorString();
+//      qDebug() << "Payload file " << fileName << " could not be open for reading!";
+//      qDebug() << "Error: " << file.errorString();
       return QByteArray();
     }
   } else
@@ -255,7 +252,7 @@ QByteArray PartHelper::translateData( qint64 id, const QByteArray &data, bool is
 
 QByteArray PartHelper::translateData( const Part& part )
 {
-  return translateData( part.id(), part.data(), part.external() );
+  return translateData( part.data(), part.external() );
 }
 
 /** Returns the record with id @p id. */
